package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.storage_predicted_power_data.DynamicQueryStoragePredictedPowerInput;
import pps.cloud.space.service.ISpaceOptimizeLongCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.SpaceInstitutionDetailEnt;
import pps.core.space.entity.SpaceOptimizeDurationDTO;
import pps.core.space.entity.SpaceOptimizePeriodDTO;
import pps.core.space.entity.SpaceOptimizeWellheadDTO;
import pps.core.space.mapper.SpaceOptimizeViewMapper;
import pps.core.space.service.data.SpaceOptimizeLineRelation;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.util.ArrayList;
import java.util.List;

/**
 * 长期间开优化Cloud模块
 *
 * @author ZWT
 * @date 2023/09/13 14:14
 */
@XService
public class SpaceOptimizeLongCloudServiceImpl extends SpaceOptimizeBaseService implements ISpaceOptimizeLongCloudService {

    /**
     * 长期间开优化Cloud模块--定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult optimizeLongJob(XContext context) {
        //取当前时间
        DateTime date = DateUtil.date();
        //取生效中的基础间开
        List<SpaceInstitutionDetailEnt> detailEntList = super.getEffectiveSpaceInstitution(context, date.toString(BusinessConstant.DATE_FORMAT_DAY));
        SpaceOptimizeLineRelation relation = super.getOptimizeLineRelation(context, detailEntList);
        //取年份
        int year = date.year();
        //月份要加一
        int monthIndex = date.month();
        int monthNum = monthIndex + 1;
        int lastMonthNum = monthIndex;
        int lastYear = year;
        if (monthIndex == 0) {
            lastMonthNum = 12;
            lastYear = year - 1;
        }
        String month = BaseUtils.getMonthString(monthNum);
        String lastMonth = BaseUtils.getMonthString(lastMonthNum);
        List<SpaceOptimizePeriodDTO> periodDTOList = new ArrayList<>(32);
        List<SpaceOptimizeWellheadDTO> wellheadDTOList = new ArrayList<>(64);
        List<SpaceOptimizeDurationDTO> durationDTOList = new ArrayList<>(128);
        List<SpaceOptimizeDurationDTO> unOptimizeDurationList = new ArrayList<>(128);
        String executionCycleForMonth = BaseUtils.getExecutionCycleForMonth(date);
        DateTime optimizeDeadline = DateUtil.endOfMonth(date);
        //优化
        for (SpaceInstitutionDetailEnt detail : detailEntList) {
            //创建记录
            String periodId = super.createOptimizePeriod(periodDTOList, detail.getId(), detail.getLineId(), executionCycleForMonth, optimizeDeadline);
            switch (detail.getGridTypeKey()) {
                //并网型优化
                case "1":
                    super.gridConnectedOptimization(context, wellheadDTOList, durationDTOList, unOptimizeDurationList,
                            relation.getWellheadsByInstitutionIdIdMap().get(detail.getId()), detail, monthNum, periodId,
                            DynamicQueryPlantPredictedPowerInput.builder()
                                    .plantIds(relation.getPlantIdsByLineIdMap().get(detail.getLineId()))
                                    .yearTime(String.valueOf(year))
                                    .monthTime(month)
                                    .dateType(BusinessConstant.ZERO)
                                    .build());
                    break;
                //离网型优化
                case "0":
                    super.offGridOptimization(context, wellheadDTOList, durationDTOList, unOptimizeDurationList, relation.getWellheadsByInstitutionIdIdMap().get(detail.getId()),
                            relation.getStorageAvgMap(), detail, periodId,
                            DynamicQueryStoragePredictedPowerInput.builder()
                                    .storageIds(relation.getPlantIdsByLineIdMap().get(detail.getLineId()))
                                    .yearTime(String.valueOf(lastYear))
                                    .monthTime(lastMonth)
                                    .build(),
                            DynamicQueryPlantPredictedPowerInput.builder()
                                    .plantIds(relation.getPlantIdsByLineIdMap().get(detail.getLineId()))
                                    .yearTime(String.valueOf(year))
                                    .monthTime(month)
                                    .dateType(BusinessConstant.ZERO)
                                    .build());
                    break;
                default:
                    //电网类型不存在
            }
        }
        //开启事务
        return XTransactionHelper.begin(context, () -> {
            int size;
            SpaceOptimizeViewMapper optimizeViewMapper = context.getBean(SpaceOptimizeViewMapper.class);
            if (CollUtil.isNotEmpty(periodDTOList)) {
                size = periodDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizePeriodDTO>> subList = BaseUtils.getSubList(periodDTOList);
                    subList.forEach(b -> optimizeViewMapper.longPeriodBatchInsertList(b));
                } else {
                    optimizeViewMapper.longPeriodBatchInsertList(periodDTOList);
                }
            }
            if (CollUtil.isNotEmpty(wellheadDTOList)) {
                size = wellheadDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeWellheadDTO>> subList = BaseUtils.getSubList(wellheadDTOList);
                    subList.forEach(b -> optimizeViewMapper.longWellheadBatchInsertList(b));
                } else {
                    optimizeViewMapper.longWellheadBatchInsertList(wellheadDTOList);
                }
            }
            if (CollUtil.isNotEmpty(unOptimizeDurationList)) {
                size = unOptimizeDurationList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(unOptimizeDurationList);
                    subList.forEach(b -> optimizeViewMapper.longDurationBatchInsertList(b));
                } else {
                    optimizeViewMapper.longDurationBatchInsertList(unOptimizeDurationList);
                }
            }
            if (CollUtil.isNotEmpty(durationDTOList)) {
                size = durationDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(durationDTOList);
                    subList.forEach(b -> optimizeViewMapper.longDurationBatchInsertList(b));
                } else {
                    optimizeViewMapper.longDurationBatchInsertList(durationDTOList);
                }
            }
            return XServiceResult.OK;
        });
    }
}